package apap.tutorial.belajarbelajar.controller;

import apap.tutorial.belajarbelajar.model.PengajarModel;
import apap.tutorial.belajarbelajar.model.PenyelenggaraModel;
import apap.tutorial.belajarbelajar.service.PenyelenggaraService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Controller;
import org.springframework.stereotype.Repository;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.*;

import apap.tutorial.belajarbelajar.model.CourseModel;
import apap.tutorial.belajarbelajar.service.CourseService;

import java.util.ArrayList;
import java.util.List;

@Controller
public class CourseController {
    @Qualifier("courseServiceImpl")
    @Autowired
    private CourseService courseService;

    @Autowired
    private PenyelenggaraService penyelenggaraService;

    @Autowired
    private PenyelenggaraService pengajarService;

    @GetMapping("course/add")
    public String addCourseFormPage(Model model) {
        CourseModel course = new CourseModel();
        List<PenyelenggaraModel> listPenyelenggara = penyelenggaraService.getListPenyelenggara();
        List<PenyelenggaraModel> listPenyelenggaraNew = new ArrayList<>();
        List<PengajarModel> listPengajarNew = new ArrayList<>();

        course.setListPenyelenggara(listPenyelenggaraNew);
        course.setListPengajar(listPengajarNew);

        course.getListPengajar().add(new PengajarModel());
        course.getListPenyelenggara().add(new PenyelenggaraModel());

        model.addAttribute("course", new CourseModel());
        model.addAttribute("listPenyelenggaraExisting", listPenyelenggara);
        return "form-add-course";
    }

    @PostMapping("course/add")
    public String addCourseSubmitPage(@ModelAttribute CourseModel course, Model model) {
        courseService.addCourse(course);
        model.addAttribute("code", course.getCode());
        return "add-course";
    }

    @GetMapping("course/viewall")
    public String listCourse(Model model) {
        List<CourseModel> listCourse = courseService.getListCourse();
        model.addAttribute("listCourse", listCourse);
        return "viewall-course";
    }

    @GetMapping("course/viewallorderbynamecourse")
    public String listCourseOrder(Model model) {
        List<CourseModel> listCourse = (List<CourseModel>) courseService.getCourseOrderByName();
        model.addAttribute("listCourse", listCourse);
        return "viewall-course";
    }

    @GetMapping("course/view")
    public String viewDetailCoursePage(@RequestParam(value = "code") String code, Model model) {
        CourseModel course = courseService.getCourseByCodeCourse(code);
        if (course != null) {
            List<PengajarModel> listPengajar = course.getListPengajar();
            List<PenyelenggaraModel> listPenyelenggara = course.getListPenyelenggara();
            model.addAttribute("listPengajar", listPengajar);
            model.addAttribute("listPenyelenggara", listPenyelenggara);
            model.addAttribute("course", course);
            return "view-course";
        }
        else {
            return  "cant-find-course";
        }
    }

    @GetMapping(value = "/course/view-query")
    public String viewDetailCoursePageQuery(@RequestParam(value = "code") String code, Model model) {
        CourseModel course = courseService.getCourseByCodeCourseQuery(code);
        List<PengajarModel> listPengajar = course.getListPengajar();
        model.addAttribute("listPengajar", listPengajar);
        model.addAttribute("course", course);

        return "view-course";
    }

    @GetMapping(value = "/course/update/{code}")
    public String updateCourseFormPage(@PathVariable String code, Model model) {

        CourseModel course = courseService.getCourseByCodeCourse(code);
        model.addAttribute("course", course);

        return "form-update-course";
    }

    @PostMapping(value = "/course/update")
    public String updateCourseSubmitPage(@ModelAttribute CourseModel course, Model model) {
        CourseModel updatedCourse = courseService.updateCourse(course);
        model.addAttribute("code", updatedCourse.getCode());

        return "update-course";
    }

    @GetMapping(value = "/course/viewsortedlistcourse")
    public String viewSortedList(Model model) {
        List<CourseModel> listCourse = courseService.getListCourseOrderByNameCourse();
        model.addAttribute("listCourse", listCourse);
        return "viewall-sorted-course";
    }

    @GetMapping(value = "/course/delete/{code}")
    public String deleteCourse(@PathVariable String code, Model model) {
        CourseModel course = courseService.getCourseByCodeCourse(code);
        List<PengajarModel> listPengajar = course.getListPengajar();
        boolean isEnding = courseService.compareDateTime(course.getTanggalBerakhir(), course.getTanggalDimulai());

        if(!isEnding || !listPengajar.isEmpty()) {
            return "cant-delete-course";
        }
        else {
            courseService.deleteCourse(course, course.getTanggalBerakhir(), course.getTanggalDimulai());
        }
        model.addAttribute("course", course);

        return "delete-course";
    }

    @PostMapping(value = "/course/add", params = {"addRow"})
    private String addRowCourseMultiple(
            @ModelAttribute CourseModel course,
            Model model
    ){
        if (course.getListPenyelenggara() == null || course.getListPenyelenggara().size() == 0) {
            course.setListPenyelenggara(new ArrayList<>());
        }
        course.getListPenyelenggara().add(new PenyelenggaraModel());
        List<PenyelenggaraModel> listPenyelenggara = penyelenggaraService.getListPenyelenggara();

        model.addAttribute("course", course);
        model.addAttribute("listPenyelenggaraExisting", listPenyelenggara);

        return "form-add-course";
    }

    @PostMapping(value = "/course/add", params = {"addRow2"})
    private String addRowCourseMultiplePengajar(
            @ModelAttribute CourseModel course,
            Model model
    ) {
        if (course.getListPengajar() == null || course.getListPengajar().size() == 0) {
            course.setListPengajar(new ArrayList<>());
        }

        course.getListPengajar().add(new PengajarModel());
        List<PenyelenggaraModel> listPenyelenggara = penyelenggaraService.getListPenyelenggara();

        model.addAttribute("course", course);
        model.addAttribute("listPenyelenggaraExisting", listPenyelenggara);

        return "form-add-course";
    }

    @PostMapping(value = "/course/add", params = {"deleteRow"})
    private String deleteRowCourseMultiple(
            @ModelAttribute CourseModel course,
            @RequestParam("deleteRow") Integer row,
            Model model
    ){
        final Integer rowId = Integer.valueOf(row);
        course.getListPenyelenggara().remove(rowId.intValue());

        List<PenyelenggaraModel> listPenyelenggara = penyelenggaraService.getListPenyelenggara();

        model.addAttribute("course", course);
        model.addAttribute("listPenyelenggaraExisting", listPenyelenggara);

        return "form-add-course";
    }

    @PostMapping(value = "/course/add", params = {"deleteRow2"})
    private String deleteRowCourseMultiplePengajar(
            @ModelAttribute CourseModel course,
            @RequestParam("deleteRow2") Integer row,
            Model model
    ) {
        final Integer rowId = Integer.valueOf(row);
        course.getListPengajar().remove(rowId.intValue());

        List<PenyelenggaraModel> listPenyelenggara = penyelenggaraService.getListPenyelenggara();

        model.addAttribute("listPenyelenggaraExisting", listPenyelenggara);
        model.addAttribute("course", course);

        return "form-add-course";
    }


    @PostMapping(value = "/course/add", params = {"save"})
    private String addCourseSubmit(
            @ModelAttribute CourseModel course,
            Model model
    ){
        if (course.getListPenyelenggara() == null) {
            course.setListPenyelenggara(new ArrayList<>());
        }
        for (PengajarModel pengajar : course.getListPengajar()) {
            pengajar.setCourse(course);
        }
        courseService.addCourse(course);
        System.out.println(course.getCode());
        model.addAttribute("code", course.getCode());
        return "add-course";
    }
}