package apap.tutorial.belajarbelajar.controller;

import apap.tutorial.belajarbelajar.model.CourseModel;
import apap.tutorial.belajarbelajar.model.PengajarModel;
import apap.tutorial.belajarbelajar.service.CourseService;
import apap.tutorial.belajarbelajar.service.PengajarService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;

import java.util.ArrayList;
import java.util.List;

@Controller
public class PengajarController {
    @Qualifier("pengajarServiceImpl")
    @Autowired
    private PengajarService pengajarService;

    @Qualifier("courseServiceImpl")
    @Autowired
    private CourseService courseService;

    @GetMapping("/pengajar/add/{code}")
    public String addPengajarFormPage(@PathVariable String code, Model model) {
        PengajarModel pengajar = new PengajarModel();
        CourseModel course = courseService.getCourseByCodeCourse(code);
        pengajar.setCourse(course);
        model.addAttribute("pengajar", pengajar);
        return "form-add-pengajar";
    }

    @PostMapping("/pengajar/add")
    public String addPengajarSubmitPage(@ModelAttribute PengajarModel pengajar, Model model) {
        pengajarService.addPengajar(pengajar);
        model.addAttribute("noPengajar", pengajar.getNoPengajar());
        return "add-pengajar";
    }

    @GetMapping(value = "/pengajar/update/{noPengajar}")
    public String updatePengajarFormPage(@PathVariable Long noPengajar, Model model) {
        PengajarModel pengajar = pengajarService.getPengajarByNoPengajar(noPengajar);
        boolean isEnding = pengajarService.compareDateTime(pengajar.getCourse().getTanggalBerakhir(), pengajar.getCourse().getTanggalDimulai());

        if(!isEnding) {
            return "cant-update-pengajar";
        }
        model.addAttribute("pengajar", pengajar);

        return "form-update-pengajar";
    }

    @PostMapping(value = "/pengajar/update")
    public String updateCourseSubmitPage(@ModelAttribute PengajarModel pengajar, Model model) {
        PengajarModel updatedPengajar = pengajarService.updatePengajar(pengajar);
        model.addAttribute("noPengajar",updatedPengajar.getNoPengajar());

        return "update-pengajar";
    }

//    @GetMapping(value = "/pengajar/delete/{noPengajar}")
//    public String deletePengajar(@PathVariable Long noPengajar, Model model) {
//        PengajarModel pengajar = pengajarService.getPengajarByNoPengajar(noPengajar);
//        boolean isEnding = pengajarService.compareDateTime(pengajar.getCourse().getTanggalBerakhir(), pengajar.getCourse().getTanggalDimulai());
//
//        System.out.println(isEnding);
//
//        if(!isEnding) {
//            System.out.println("Harusnya masuknya disini");
//            return "cant-delete-pengajar";
//        }
//        else {
//            pengajarService.deletePengajar(pengajar, pengajar.getCourse().getTanggalBerakhir(), pengajar.getCourse().getTanggalDimulai());
//        }
//        model.addAttribute("pengajar", pengajar);
//
//        return "delete-pengajar";
//    }

    @PostMapping("/pengajar/delete")
    public String deletePengajarSubmit(
            @ModelAttribute CourseModel course,
            Model model
    ){
        if (courseService.isClosed(course.getTanggalDimulai(), course.getTanggalBerakhir())) {
            for (PengajarModel pengajar :
                course.getListPengajar()) {
                pengajarService.deletePengajar(pengajar, pengajar.getCourse().getTanggalDimulai(), pengajar.getCourse().getTanggalBerakhir());
            }
            model.addAttribute("code", course.getCode());
            return "delete-pengajar";
        }
        System.out.println("Masuk Sini");
        return "cant-delete-pengajar";
    }
}